//=============================================================================
//  Easy Clarinet Alternate Fingering Plugin
//
//  Copyright (C) 2022 by Babecka
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2
//  as published by the Free Software Foundation and appearing in
//  the file LICENCE.GPL
//=============================================================================

import QtQuick 2.0
import MuseScore 3.0

MuseScore {
	version: "1.0"
	description: qsTr("This plugin adds fingerings for easy alternate clarinet by Babecka")
	menuPath: qsTr("Plugins.Clarinet Easy Alt")

	Component.onCompleted : {
		if (mscoreMajorVersion >= 4) {
			title = "Clarinet Easy Alt";
		}
	}

	
	property variant seperator: "\n";
	property variant offsetFactor: -2.3;
	
	property variant verbose: false;
	
	property variant baseKey: -2;

	property variant noteTable: {
		
	
		48: {fingering: "3\n+\nP2"}, // D
		49: {fingering: "3\n+\nP1"}, // Eb
		50: {fingering: ""}, // E
		51: {fingering: "3\n+\n1\n2\n3"}, // F
		52: {fingering: ""}, // Gb
		53: {fingering: "3\n+\n1\n2"}, // G
		54: {fingering: "3\n+\n1"}, // Ab
		55: {fingering: "3\n+\n2"}, // A
		56: {fingering: "3"}, // Bb - C clar
		57: {fingering: ""}, // B
		58: {fingering: "2"}, // C <-----
		59: {fingering: ""}, // Db
		60: {fingering: "1"}, // D
		61: {fingering: "T"}, // Eb
		62: {fingering: "1f"}, // E
		63: {fingering: "0"}, // F
		64: {fingering: ""}, // Gb
		65: {fingering: "A"}, // G
		66: {fingering: "Bb"}, // Ab
		67: {fingering: "2T3\n+\nP2"}, // A
		68: {fingering: "2T3\n+\nP1"}, // Bb
		69: {fingering: ""}, // B
		70: {fingering: "2T3\n+\n1\n2\n3"}, // C
		71: {fingering: ""}, // Db
		72: {fingering: "2T3\n+\n1\n2"}, // D
		73: {fingering: "2T3\n+\n1"}, // Eb
		74: {fingering: ""}, // E
		75: {fingering: "2T3"}, // F
		76: {fingering: ""}, // Gb
		77: {fingering: "2T2"}, // G
		78: {fingering: ""}, // Ab
		79: {fingering: "2T1"}, // A
		80: {fingering: "2T"}, // Bb
		
	}
	
	function getNoteName (tpc) {
		var noteName = "";
		switch (tpc) {
			case -1: noteName = qsTranslate("InspectorAmbitus", "Fbb") + noteName; break;
			case  0: noteName = qsTranslate("InspectorAmbitus", "Cbb") + noteName; break;
			case  1: noteName = qsTranslate("InspectorAmbitus", "Gbb") + noteName; break;
			case  2: noteName = qsTranslate("InspectorAmbitus", "Dbb") + noteName; break;
			case  3: noteName = qsTranslate("InspectorAmbitus", "Abb") + noteName; break;
			case  4: noteName = qsTranslate("InspectorAmbitus", "Ebb") + noteName; break;
			case  5: noteName = qsTranslate("InspectorAmbitus", "Bbb") + noteName; break;
			case  6: noteName = qsTranslate("InspectorAmbitus", "Fb")  + noteName; break;
			case  7: noteName = qsTranslate("InspectorAmbitus", "Cb")  + noteName; break;

			case  8: noteName = qsTranslate("InspectorAmbitus", "Gb")  + noteName; break;
			case  9: noteName = qsTranslate("InspectorAmbitus", "Db")  + noteName; break;
			case 10: noteName = qsTranslate("InspectorAmbitus", "Ab")  + noteName; break;
			case 11: noteName = qsTranslate("InspectorAmbitus", "Eb")  + noteName; break;
			case 12: noteName = qsTranslate("InspectorAmbitus", "Bb")  + noteName; break;
			case 13: noteName = qsTranslate("InspectorAmbitus", "F")   + noteName; break;
			case 14: noteName = qsTranslate("InspectorAmbitus", "C")   + noteName; break;
			case 15: noteName = qsTranslate("InspectorAmbitus", "G")   + noteName; break;
			case 16: noteName = qsTranslate("InspectorAmbitus", "D")   + noteName; break;
			case 17: noteName = qsTranslate("InspectorAmbitus", "A")   + noteName; break;
			case 18: noteName = qsTranslate("InspectorAmbitus", "E")   + noteName; break;
			case 19: noteName = qsTranslate("InspectorAmbitus", "B")   + noteName; break;

			case 20: noteName = qsTranslate("InspectorAmbitus", "F♯")  + noteName; break;
			case 21: noteName = qsTranslate("InspectorAmbitus", "C♯")  + noteName; break;
			case 22: noteName = qsTranslate("InspectorAmbitus", "G♯")  + noteName; break;
			case 23: noteName = qsTranslate("InspectorAmbitus", "D♯")  + noteName; break;
			case 24: noteName = qsTranslate("InspectorAmbitus", "A♯")  + noteName; break;
			case 25: noteName = qsTranslate("InspectorAmbitus", "E♯")  + noteName; break;
			case 26: noteName = qsTranslate("InspectorAmbitus", "B♯")  + noteName; break;
			case 27: noteName = qsTranslate("InspectorAmbitus", "F♯♯") + noteName; break;
			case 28: noteName = qsTranslate("InspectorAmbitus", "C♯♯") + noteName; break;
			case 29: noteName = qsTranslate("InspectorAmbitus", "G♯♯") + noteName; break;
			case 30: noteName = qsTranslate("InspectorAmbitus", "D♯♯") + noteName; break;
			case 31: noteName = qsTranslate("InspectorAmbitus", "A♯♯") + noteName; break;
			case 32: noteName = qsTranslate("InspectorAmbitus", "E♯♯") + noteName; break;
			case 33: noteName = qsTranslate("InspectorAmbitus", "B♯♯") + noteName; break;
			default: noteName = qsTr("?")   + noteName; break;
		} 
		return noteName;
   }
   
   function log(msg) {
		if (verbose)
			console.log(msg);
	}
	
	function getFingering(instrument, notes) {
		var sections = instrument.split(".");
		if (sections == "undefined" || sections.length < 1)
			return "";
		var mode = sections[1] == "trombone" ? "slide" : "fingering";
		
		var text = "";
		for (var i = 0; i < notes.length; i++) {
			if (text != "")
				text += seperator
			
			var index = notes[i].pitch + baseKey;
			log("Pitch: " + index + " | NoteName: " + getNoteName(notes[i].tpc));
			if (index in noteTable  && notes[i].tieBack == null)
				if (mode in noteTable[index])
					text += qsTr(noteTable[index][mode]);
		}
		return text;
	}
	
	onRun: {
		if (typeof curScore === 'undefined')
			Qt.quit();
		var cursor = curScore.newCursor();
		var startStaff;
		var endStaff;
		var endTick;
		var fullScore = false;
		cursor.rewind(1);
		if (!cursor.segment) {
			fullScore = true;
			startStaff = 0;
			endStaff = curScore.nstaves - 1;
		} else {
			startStaff = cursor.staffIdx;
			cursor.rewind(2);
			if (cursor.tick == 0) {
				// this happens when the selection includes
				// the last measure of the score.
				// rewind(2) goes behind the last segment (where
				// there's none) and sets tick=0
				endTick = curScore.lastSegment.tick + 1;
			} else {
				endTick = cursor.tick;
			}
			endStaff = cursor.staffIdx;
		}
		
		for (var staff = startStaff; staff <= endStaff; staff++) {
			cursor.rewind(1)
			cursor.voice = 0;
			cursor.staffIdx = staff;
			
			if (fullScore)  // no selection
				cursor.rewind(0); // beginning of score
			
			var instrument = curScore.parts[staff].instrumentId;
			if (instrument.substring(0, 6) != "woodwinds.") 
			
			while (cursor.segment && (fullScore || cursor.tick < endTick)) {
				if (!cursor.element || cursor.element.type != Element.CHORD) {
					cursor.next();
					continue;
				}
				
				var fingering = getFingering(instrument, cursor.element.notes);
				if (fingering) {
					var text = newElement(Element.FIGURED_BASS);
					text.text = fingering;
					text.offsetY = (cursor.element.notes.length - 1) * offsetFactor;
					cursor.add(text);
				}
				
				cursor.next();
			}
		}
		
		(typeof(quit) === 'undefined' ? Qt.quit : quit)()

	}	
}
